---
title: Hooks
---

# Hooks

Hooks 钩子函数用于在 Global IM UIKit 需要时，从业务层获取所需的必要数据，如用户信息、群组信息等。开发者需要在初始化 Global IM UIKit 时，按要求完成必要的钩子函数实现。

:::tip

 **由于钩子函数定义较为复杂，本章内容均使用 Typescript 代码示例，以便于用户更好的理解钩子函数的参数和返回值数据结构。**
:::


## Hooks 定义

Global IM UIKit 中通过 [IRCKitServiceHooks] 接口定义所需的所有钩子，声明如下：

```ts
export interface IRCKitServiceHooks {
  /**
   * 根据 userId 批量获取用户信息
   * @param userIds - 用户 ID 列表
   */
  reqUserProfiles(userIds: string[]): Promise<IRCKitUserProfile[]>;
  /**
   * 根据 groupId 批量获取群组信息
   * @param groupIds - 群组 ID 列表
   */
  reqGroupProfiles(groupIds: string[]): Promise<IRCKitGroupProfile[]>;
  /**
   * 批量获取系统会话信息，当会话列表中存在 ConversationType.SYSTEM 类型时，会调用此方法获取系统会话信息
   * @param targetIds - 系统会话 ID 列表
   */
  reqSystemProfiles(targetIds: string[]): Promise<IRCKitSystemProfile[]>;
  /**
   * 获取群组成员信息
   * @param groupId - 群组 ID
  */
  reqGroupMembers(groupId: string): Promise<IRCKitGroupMemberProfile[]>;
  /**
    * [可选]定义默认用户信息，以替换 SDK 默认初始数据，如头像、名称规则等
    * @param userId - 用户 ID
    */
  getDefaultUserProfile?(userId: string): IRCKitUserProfile;
  /**
    * [可选]定义默认群组信息，以替换 SDK 默认数据，，如头像、名称规则等
    * @param groupId - 群组 ID
    */
  getDefaultGroupProfile?(groupId: string): IRCKitGroupProfile;
  /**
    * [可选]定义默认系统会话信息，以替换 SDK 默认数据，，如头像、名称规则等
    * @param systemId - 系统会话 ID
    */
  getDefaultSystemProfile?(systemId: string): IRCKitSystemProfile;
  /**
   * [可选]请求用户在线状态
   * @param userIds - 用户 ID 列表
   */
  reqUserOnlineStatus?(userIds: string[]): Promise<IRCKitUserOnlineStatus[]>;
}
```

## 调用与缓存

Global IM UIKit 对于从业务层获取到的用户信息、群组信息等，会在内部进行缓存，以避免重复请求。

下图以用户信息为例，展示了 Global IM UIKit 中用户信息的获取和更新机制。

```mermaid
sequenceDiagram
    participant App
    participant Kit as Global IM UIKit
    participant UI as Web Component UI 组件

    UI->>Kit: 获取所需用户信息
    Note left of UI: 出于性能考虑，UI 组件在获取信息时会尽可能批量处理
    Kit-->>Kit: 定义待请求列表 reqList，结果列表 resList
    loop 遍历请求列表 userId
        Kit-->>Kit: 根据 userId 查找缓存数据
        alt 缓存数据为有效真实数据
            Kit-->>Kit: 提取数据加入 resList
        else 缓存数据为默认数据（非真实数据）
            Kit-->>Kit: 提取数据加入 resList，并将 userId 加入 reqList
        else 无缓存数据
            Kit-->>App: getDefaultUserProfile(userId)
            Note left of UI: getDefaultUserProfile 是同步函数，用于立即获取用户信息。<br/> 未实现 getDefaultUserProfile 钩子时，SDK 将使用内置定义。
            App->>Kit: 返回默认数据
            Kit-->>Kit: 缓存数据
            Kit-->>Kit: 将数据加入 resList，并将 userId 加入 reqList
        end
    end
    Kit->>UI: 返回 resList
    Note left of Kit: 由于真实数据一半需要经过网络请求获取，耗时较长。<br/> 为提升 UI 展示效率，SDK 会先返回包含非真实数据的结果。

    UI-->>UI: 根据结果列表渲染用户信息

    Kit->>App: await reqUserProfiles(reqList)
    Note right of App: 通过 reqUserProfiles 钩子获取真实数据<br/> 您可以能需要在此处进行网络请求，所以 reqUserProfiles 钩子是个异步函数

    App-->>App: 等待服务数据返回
    App->>Kit: 返回 promise 结果
    Kit-->>Kit: 校验结果并更新缓存数据
    Kit->>UI: 通知 UI 更新用户信息
    UI-->>UI: 更新用户信息
```

当用户信息、群组信息等发生变化时，业务层可以调用 `updateUserProfile` 等方法通知 Global IM UIKit 更新缓存数据，并及时更新 UI 显示。详情参考[数据更新](./update)章节。

```mermaid
sequenceDiagram
    participant App
    participant Kit as Global IM UIKit
    participant UI as Web Component UI 组件

    App->>Kit: 调用 updateUserProfile 更新用户信息
    Kit-->>Kit: 校验并更新缓存数据
    Kit->>UI: 通知用户信息变更
```

## 代码示例

```ts
import { IRCKitServiceHooks, RCKitInstaller } from '@rongcloud/global-im-uikit';
// 您也可以通过如下方式进行 Global IM UIKit 引用，为便于后续理解文档示例，此处使用 import * as RCIMKit 方式引用
// import { RCKitInstaller } from '@rongcloud/global-im-uikit';

const hooks: IRCKitServiceHooks = {
  // 定义人员信息获取方法
  async reqUserProfiles(userIds: string[]): Promise<IRCKitUserProfile[]> {
    // userIds 为用户 ID 列表，业务层需要根据 userIds 提供对应的用户信息
    return userIds.map((userId) => ({
      userId,
      name: '', // 用户名称，请传递有效值，否则会影响用户信息展示
      portraitUri: '', // 用户头像，请传递有效值，当头像获取失败或未提供，SDK 将使用默认头像
    }));
  },

  // 定义群组信息获取方法
  async reqGroupProfiles(groupIds: string[]): Promise<IRCKitGroupProfile[]> {
    // groupIds 为群组 ID 列表，业务层需要根据 groupIds 提供对应的群组信息
    return groupIds.map((groupId) => ({
      groupId,
      name: '', // 群组名称，请传递有效值，否则会影响群组信息展示
      memberCount: 0, // 群成员数量，请传递有效值，否则会影响群组信息展示
      portraitUri: '', // 群组头像，请传递有效值，当头像获取失败或未提供，SDK 将使用默认头像
    }));
  },

  // 定义群组成员信息获取方法
  async reqGroupMembers(groupId: string): Promise<IRCKitGroupMemberProfile[]> {
    // groupId 为群组 ID，业务层需要根据 groupId 提供对应的群组成员信息
    return [
      // 需要业务层提供群组的真实成员信息，其中 nickname 为可选数据
      // nickname 为空时，SDK 将通过 reqUserProfiles 获取的用户信息中的 name 字段作为群组成员的昵称
      // 从性能考虑，建议业务层提供群组成员的真实昵称，以避免 SDK 内部多次调用 reqUserProfiles 方法
      { userId: 'user-01', nickname: '' },
      { userId: 'user-02' }
    ]
  },

  // 定义系统会话信息获取方法
  async reqSystemProfiles(systemIds: string[]): Promise<IRCKitSystemProfile[]> {
    // systemIds 为系统会话 ID 列表，业务层需要根据 systemIds 提供对应的系统会话信息
    return systemIds.map((systemId) => ({
      systemId,
      name: '', // 系统会话名称，请传递有效值，否则会影响系统会话信息展示
      portraitUri: '', // 系统会话头像，请传递有效值，当头像获取失败或未提供，SDK 将使用默认头像
    }));
  },
}

// 初始化 IMKit，将 hooks 作为参数传入
const kitApp = RongIMLib.installPlugin(RCKitInstaller, { hooks });
```

[IRCKitServiceHooks]: https://doc.rongcloud.cn/apidoc/im-uikit-web/latest/zh_CN/interfaces/IRCKitServiceHooks.html

